class Connect4:
    """This implements a game of Connect4"""
    def __init__(self, state=None, players=None):
        #Game Constants
        self.dims = (7 , 6)

        if state:
            self.state = state
        else:
            self.state = {
                "turn": 0,
                "players": players,
                "board": [[0]*self.dims[1] for _ in range(self.dims[0])]
            }

    def print(self):
        print("---------------")
        for j in range(self.dims[1]):
            pStr = "|"
            for i in range(self.dims[0]):
                pStr += str(self.state["board"][i][self.dims[1] - 1 - j]) + "|"
            print(pStr)
        print("---------------")

    def validateMove(self, move, player):
        if move < 0 or move >= self.dims[0]:
            return False

        if self.state["board"][move][-1] == 0:
            return True

        return False

    def makeMove(self, move, player):#assumes move has been validated
        y = self.dims[1] - 1
        while self.state["board"][move][y] == 0:
            y += -1
            if y < 0:
                break

        marker = self.state["turn"] + 1
        self.state["board"][move][y+1] = marker


    def postMove(self):
        dirs = [(0,1),(1,0),(1,1),(-1,1)]
        #check for winner
        for i in range(self.dims[0]):
            j = 0
            while self.state["board"][i][j] != 0:
                match = self.state["board"][i][j]
                for dir in dirs:
                    streak = 1
                    dx = i + dir[0]
                    dy = j + dir[1]
                    if dx >= self.dims[0] or dy >= self.dims[1]:
                        continue
                    while self.state["board"][dx][dy] == match:
                        streak += 1
                        if streak == 4:
                            self.endGame(match - 1)
                        dx += dir[0]
                        dy += dir[1]

                        if dx >= self.dims[0] or dy >= self.dims[1]:
                            break

                j += 1
                if j == self.dims[1]:
                    break

        #if no winner incement move
        self.state["turn"] = (self.state["turn"] + 1) % len(self.state["players"])

    def endGame(self, winner):
        #print("Winner", winner)
        #print(self.state["players"][winner])
        self.state["Winner"] = winner



import AIArena.games
from AIArena.AI import AI as AI
from AIArena.Ref import Ref as Ref

from AIArena.Human import Human_Connect4 as Human_Connect4

import pkgutil
import requests
import os
from brainiac_client.scientist import package


import json

name = "AIArena"

def runGame(Ais,game):
    gameName = game["name"]
    result = {
        "error":"None",
        "version":20
    }
    #Game setup
    try:
        ref = Ref()
        ref.createGame(gameName, Ais)
    except:
        result["error"] = "Game Creation"
        return result

    #Game run
    try:
        result["Winner"] = ref.runGame()
        if result["Winner"] == -1:
            result["error"] = "Game Run"
    except:
        result["error"] = "Game Run"
        return result

    #Game successfully finished
    result["moves"] = ref.moves
    result["states"] = ref.states
    return result

def runMove(AIs, data):
    try:
        AI = AIs[0]
        state = json.loads(data["gamestate"])
        move = AI.makeMove(state)

        results = {
            "error": "None"
        }

        if move == None:
            results['error'] = 'Bad move response'
        else:
            results['move'] = move

        return results
    except Exception as e:
        print("EXCEPTION:", e)
        return {'exception': e}


def upload(AI, aid):
    #os.environ['BRAINIAC_CLIENT_USERNAME'] = "ai_olympics"
    #os.environ['BRAINIAC_CLIENT_PASSWORD'] = "ai_Olympics_brainiac_#1"
    os.environ['BRAINIAC_CLIENT_USERNAME'] = "AIArena"
    os.environ['BRAINIAC_CLIENT_PASSWORD'] = "Letsbrainiac!1"
    #brainiac = BrainiacClient(http=True)
    response = package(AI, aid)
    if response.status_code != 200:
        print("An error occurred when trying to upload:",response.status_code)
        return
    print("AI Packaged Successfully")
    payload = {
        "aid":aid,
        "game":AI.game
    }

    r = requests.get("https://us-central1-ai-olympics.cloudfunctions.net/newAiUpload",payload)
    #print(r.status_code)
    #print(r.headers)
    if r.status_code == 200:
        print(r.content.decode())
    else:
        print("A registration error occured:", r.status_code, r.content)


import requests
import json

APIKey = "0C0G6TQ153Q7Y67L"




class StockMarket:
    """This implements a game of a Stock Market Simulator"""
    def __init__(self, state=None, players=None):
        #Game Constants
        self.startValue = 10**6
        self.interval = 15

        if state:
            self.state = state
        else:
            self.state = {
                "endDate": "endDate",
                "symbols": ["FB"],#,"AMZN","AAPL","NFLX","GOOGL"],
                "data":{},
                "players": players,
                "portfolios":{}
            }

            for s in self.state["symbols"]:
                self.state["data"][s] = {
                    "all":{},
                    "last":{}
                }

            for p in players:
                self.state["portfolios"][p.id] = {
                    "cash":self.startValue,
                    "stocks":{},
                    "trades":[]
                }

    def print(self):
        print("---------------")
        for id, port in self.state["portfolios"].items():
            print(id)
            print(port['cash'])
            print(port['stocks'])
        print("---------------")

    def gatherStockData(self):
        for sym in self.state["symbols"]:
            data, last = self.getStockData(sym, str(self.interval)+"min")
            for date, value in data.items():
                self.state["data"][sym]["all"][date] = {}
                for key, val in value.items():
                    self.state["data"][sym]["all"][date][key[3:]] = float(val)
            self.state["data"][sym]["last"] = self.state["data"][sym]["all"][last]
            print(self.state["data"][sym])

    def getStockData(self, symbol, interval):
        payload = {
            "function": "TIME_SERIES_INTRADAY",
            "symbol": symbol,
            "interval": interval,
            "apikey": APIKey
        }

        r = requests.get("https://www.alphavantage.co/query", payload)
        if r.status_code == 200:
            data = json.loads(r.content.decode())
            last = data["Meta Data"]["3. Last Refreshed"]
            return data["Time Series (" + interval + ")"], last
        else:
            print("An error occured:", r.status_code, r.content)

    def validateMove(self, move):
        pass


    def validateTrade(self, trade, portfolio):
        if trade["symbol"] in self.state["symbols"]:
            if trade["action"] == "buy":
                cost = trade["quantity"] * self.state["data"][trade["symbol"]]["last"]["close"]
                if cost < portfolio['cash']:
                    return True
            elif trade["action"] == "sell":
                if trade["symbol"] in portfolio["stocks"] and trade["quantity"] <= portfolio["stocks"][trade["symbol"]]:
                    return True
        return False

    def makeTrade(self, trade, portfolio, aid):#Assumes trade was validated
        cost = trade["quantity"] * self.state["data"][trade["symbol"]]["last"]["close"]
        if trade["action"] == "buy":
            self.state['portfolios'][aid]['cash'] -= cost
            if trade["symbol"] in portfolio:
                self.state['portfolios'][aid]['stocks'][trade["symbol"]] += trade["quantity"]
            else:
                self.state['portfolios'][aid]['stocks'][trade["symbol"]] = trade["quantity"]
        elif trade["action"] == "sell":
            self.state['portfolios'][aid]['cash'] += cost
            self.state['portfolios'][aid]['stocks'][trade["symbol"]] -= trade["quantity"]

    def makeMove(self, move):#assumes move has been validated
        for trade in move['trades']:
            portfolio = self.state['portfolios'][move['aid']]
            if self.validateTrade(trade, portfolio):
                self.makeTrade(trade,portfolio,move['aid'])


    def postMove(self):
        pass

    def endGame(self, winner):
        #print("Winner", winner)
        #print(self.state["players"][winner])
        self.state["Winner"] = winner

import AIArena

class lilStockBuddy(AIArena.AI):#<2>
    def __init__(self, name):
        super().__init__(name, "StockMarket")

if __name__ == "__main__":
    ai1 = lilStockBuddy("lilStockBuddy")
    sm = StockMarket(players=[ai1])
    sm.gatherStockData()

    trades = [
        {'action':"buy",'symbol':"FB",'quantity':100}
    ]

    move = {
        'aid': ai1.id,
        'trades': trades
    }

    sm.print()
    sm.makeMove(move)
    sm.print()

    trades = [
        {'action': "sell", 'symbol': "FB", 'quantity': 100},
        {'action': "buy", 'symbol': "FB", 'quantity': 1000},
        {'action': "sell", 'symbol': "FB", 'quantity': 100}
    ]

    move = {
        'aid': ai1.id,
        'trades': trades
    }

    sm.makeMove(move)
    sm.print()